package com.gzlh.event;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.config.ModuleEnum;
import com.gzlh.dto.EventDTO;
import com.gzlh.infrared.config.RedPropertiesConfig;
import com.gzlh.infrared.factory.RedFactory;
import com.gzlh.led.factory.LedFactory;
import com.gzlh.led.properties.LedPropertiesConfig;
import com.gzlh.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.weighbridge.factory.WeighbridgeFactory;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Slf4j
public class EventThread implements Runnable{
    private List<EventDTO> eventDTOList;
    private String eventName;
    private final LedFactory ledFactory= SpringUtil.getBean(LedFactory.class);
    private final RedFactory redFactory=SpringUtil.getBean(RedFactory.class);
    private  final WeighbridgeFactory weighbridgeFactory=SpringUtil.getBean(WeighbridgeFactory.class);
    private final WeighbridgePropertiesConfig weighbridgePropertiesConfig=SpringUtil.getBean(WeighbridgePropertiesConfig.class);
    private final RedPropertiesConfig redPropertiesConfig=SpringUtil.getBean(RedPropertiesConfig.class);
    private final LedPropertiesConfig ledPropertiesConfig=SpringUtil.getBean(LedPropertiesConfig.class);

    public EventThread(String eventName) {
        this.eventName=eventName;
    }

    public void setEventDTOList(List<EventDTO> eventDTOList) {
        this.eventDTOList = eventDTOList;
    }
    public void setEventName(String eventName) {
        this.eventName = eventName;
    }
    /**
     * 处理事件
     *
     * @param eventName 服务.事件名 比如led.ready
     */
    public void handlerEvent(String eventName) {
        log.info("event:{}", eventName);
        eventDTOList.stream().filter(eventDTO -> StrUtil.equals(eventName, eventDTO.getName()))
                .findFirst().ifPresent(eventDTO -> {
            List<String> actionList = eventDTO.getActionList().getAction();
            //依次执行动作
            actionList.forEach(action -> {
                String module = StrUtil.subBefore(action, ".", true);
                String command = StrUtil.subAfter(action, ".", true);
                if (StrUtil.equals(module, ModuleEnum.INFRARED_MODULE.getModuleEn())) {
                    //动作属于红外
                    redFactory.handler(redPropertiesConfig.getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.LED_MODULE.getModuleEn())) {
                    //动作属于led
                    ledFactory.handler(ledPropertiesConfig.getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn())) {
                    //动作属于地磅
                    weighbridgeFactory.handler(weighbridgePropertiesConfig.getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.CENTER_MODULE.getModuleEn())) {
                    //动作属于中心总线
                    handlerAction(command);
                }
                ThreadUtil.sleep(250);
            });
        });

    }

    /**
     * 调度中心命令处理
     *
     * @param action
     */
    private void handlerAction(String action) {
        if (StrUtil.contains(action, CenterActionCommand.SLEEP_COMMAND.getCommand())) {
            //休眠
            handlerSleep(action);
        }
    }

    /**
     * 调度中心休眠
     *
     * @param action
     */
    private void handlerSleep(String action) {
        String time = StrUtil.subBefore(StrUtil.subAfter(action, "(", true), ")", true);
        if (NumberUtil.isInteger(time)) {
            ThreadUtil.sleep(Long.parseLong(time));
        }
    }

    @Override
    public void run(){
        handlerEvent(eventName);
    }
}
