package com.gzlh.gate.handler.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import cn.hutool.socket.SocketUtil;
import com.gzlh.gate.bo.ReqBO;
import com.gzlh.gate.brand.GateBrandType;
import com.gzlh.gate.handler.IGateHandler;
import com.gzlh.gate.properties.GatePropertiesConfig;
import com.gzlh.gate.task.GateBase64ToImage;
import com.gzlh.gate.utils.ZKTecoCommandUtils;
import com.gzlh.gate.utils.ZKTecoUtils;
import com.gzlh.infrared.brand.RedBrandType;
import com.gzlh.led.brand.LedBrandType;
import com.gzlh.led.client.LedNettyConfig;
import com.gzlh.led.utils.ColorConver;
import com.gzlh.led.utils.YangBandPackage;
import com.gzlh.task.TaskService;
import com.gzlh.utils.ResultJson;
import com.gzlh.utils.WordHandlerUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;

/**
 * 熵基车牌识别相机处理类
 */
@Service
@Slf4j
public class GateHandlerShangJi implements IGateHandler {
    @Resource
    private TaskService taskService;
    @Resource
    private GatePropertiesConfig gatePropertiesConfig;


    @Override
    public GateBrandType brandType() {
        return GateBrandType.SHANGJI_BRAND;
    }



    @Override
    public void handlerAction(String action) {
        log.info("action:{}",action);
    }

    /**
     * 处理相机推送请求
     * @param request 请求
     * @param response 相应
     * @return
     */
    @Override
    public ResultJson handlerReq(HttpServletRequest request, HttpServletResponse response) {
        char[] lineChars = new char[1024 * 1024];
        char[] totalChars = new char[1024 * 1024];
        int readLen = 0;
        int totalLen = 0;
        try {
            BufferedReader reader = request.getReader();
            while ((readLen = reader.read(lineChars)) > 0) {
                for (int i = 0; i < readLen; i++) {
                    totalChars[totalLen + i] = lineChars[i];
                }
                totalLen += readLen;
            }
        } catch (Exception e) {
            log.error("熵基请求异常:{}",e.getMessage());
        }
        String lineStr = new String(totalChars);
        ReqBO bo = JSONUtil.toBean(JSONUtil.parseObj(lineStr).getStr("AlarmInfoPlate"), ReqBO.class);
        String carNo= bo.getResult().getPlateResult().getLicense();
        log.info("熵基道闸请求:{}",carNo);
        taskService.addTask(new GateBase64ToImage(RandomUtil.randomNumbers(10),1000,bo.getResult().getPlateResult().getImageFile(),carNo));
        openGate(bo.getIpaddr());
        return ResultJson.success();
    }
    private void openGate(String ip){
        Socket socket=   SocketUtil.connect(ip,gatePropertiesConfig.getGatePort());
        String result = ZKTecoUtils.sendAndGetResult(socket, ZKTecoCommandUtils.openCommand());
        try {
            socket.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        Integer code = JSONUtil.parseObj(result).getInt("state_code");
        if (code == null || code != 200) {
          log.error("打开道闸失败----------");
        }
    }
}
