package com.gzlh.utils;

import cn.hutool.core.util.HexUtil;
import cn.hutool.core.util.StrUtil;
import com.gzlh.led.utils.CRC16;
import com.gzlh.led.utils.ColorConver;
import com.gzlh.led.utils.YangBandPackage;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.Charset;

public class WordHandlerUtils {


    public static String msgToASCII(String msg){
        int len=msg.length();
        StringBuilder sb=new StringBuilder();
        for (int i=0;i<len;i++){
            char ch=msg.charAt(i);
            String bit =ch+"";
            if (isChinese(ch)){//中文
                sb.append(chineseToASCII(bit));
            }else if (Character.isDigit(ch)){//数字
                sb.append(Integer.parseInt(bit) + 30);
            }else if (StrUtil.equals(".", bit)) {
                sb.append("2E");
            }
        }
        return sb.toString();
    }


    public static void main(String[] args) {
        System.out.println(msgToASCII("8222.2吨"));
        String pack=YangBandPackage.buildPackage(ColorConver.NU_TO_HEX.get(1)+msgToASCII("8222.2吨"));
        System.out.println(pack);
    }

    /**
     * 中文转ascii
     * @param zh
     * @return
     */
    private static String chineseToASCII(String zh) {
        byte[] gb2312Bytes = zh.getBytes(Charset.forName("GB2312"));
        return bytesToHexString(gb2312Bytes);
    }

    /**
     * 红色字体在文字前加代码 5C 43 31，绿色加代码5C 43 32，黄色加代码5C 43 33
     *
     * @param weight
     * @return
     */
    private static String numToASCII(int weight) {
        String str = BigDecimal.valueOf(weight).divide(BigDecimal.valueOf(1000), 2, RoundingMode.HALF_UP).toString();
        int len = str.length();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < len - 1; i++) {
            String bit = StrUtil.sub(str, i, i + 1);
            if (StrUtil.equals(".", bit)) {
                sb.append("2E").append(" ");
                continue;
            }
            sb.append(Integer.parseInt(bit) + 30).append(" ");
        }
        return  sb.toString();
    }

    private static boolean isChinese(char ch) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(ch);
        if (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B
                || ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION
                || ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS
                || ub == Character.UnicodeBlock.GENERAL_PUNCTUATION) {
            return true;
        }
        return false;
    }


    private static String bytesToHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            String hex = Integer.toHexString(b & 0xFF);
            if (hex.length() == 1) {
                sb.append('0');
            }
            sb.append(hex.toUpperCase());
        }
        return sb.toString();
    }
}

