package com.gzlh.weighbridge.handler.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.config.ApplicationConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.event.EventBus;
import com.gzlh.job.UploadTask;
import com.gzlh.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.weighbridge.event.WeighbridgeEvent;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

public class CommonWeighbridgeHandler {


    private final static List<Integer> WEIGHT_CACHE = new CopyOnWriteArrayList<>();


    public static boolean isStart = false;

    /**
     * 添加地磅数据到缓存
     *
     * @param weight 地磅数
     */
    public static void cacheWeight(int weight) {
        if (!isStart) {
            WEIGHT_CACHE.clear();
            return;
        }
        WEIGHT_CACHE.add(weight);
    }

    /**
     * 开始读取地磅数据
     */
    public static void startToRead() {
        isStart = true;
        ThreadUtil.execute(new WatchThread());
    }

    /**
     * 监控地磅缓存是否读取到了数据
     * 监控2秒内的地磅数
     */
    static class WatchThread implements Runnable {
        private int time = 3000;

        @Override
        public void run() {
            int averaWeight = 0;
            while (time > 0) {
                time = time - 1000;
                averaWeight = WEIGHT_CACHE.stream().collect(Collectors.averagingInt(Integer::intValue)).intValue();
                ThreadUtil.sleep(1000);
            }
            EventBus eventBus = SpringUtil.getBean(EventBus.class);
            WeighbridgePropertiesConfig weighbridgePropertiesConfig = SpringUtil.getBean(WeighbridgePropertiesConfig.class);
            if (averaWeight == 0 || weighbridgePropertiesConfig.getMinKg() > averaWeight) {
                //todo read timeout
                //读取超时
                eventBus.start(ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn() + "." + WeighbridgeEvent.TIMEOUT);
                return;
            }
            isStart=false;
            ApplicationConfig applicationConfig = SpringUtil.getBean(ApplicationConfig.class);
            //上传地磅数据
            String channelName = applicationConfig.getChannelName();
            String serverUrl = applicationConfig.getServerUrl();
            ThreadUtil.execute(new UploadTask(serverUrl, channelName, averaWeight));
        }
    }
}
