package com.gzlh.weighbridge.handler.impl;

import cn.hutool.core.util.StrUtil;
import com.gzlh.weighbridge.action.WeighbridgeCommonAction;
import com.gzlh.weighbridge.brand.WeighbridgeBrandType;
import com.gzlh.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.weighbridge.handler.IWeighbridgeHandler;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;

/**
 * 宁波柯力地磅处理器
 */
@Service
@Slf4j
public class WeighbridgeHandlerKeLi implements IWeighbridgeHandler {

    @Resource
    private WeighbridgePropertiesConfig weighbridgePropertiesConfig;

    @Override
    public WeighbridgeBrandType brandType() {
        return WeighbridgeBrandType.NINGBO_KELI;
    }



    @Override
    public void handlerMsg(String msg) {
        log.info(WeighbridgeBrandType.NINGBO_KELI.getBrand() + "地磅:{}", msg);
//        log.info("客户端收到消息：" + msg);
        msg = StrUtil.replace(msg, "\u0003", "").replace("\u0002", "");
//        log.info("转换后消息：" + msg);
        // 去掉开头和结尾的特殊字符
        msg = msg.substring(1, msg.length() - 1);
        // 将字符串转换成字节数组类型
        byte[] bytes = msg.getBytes();
        // 取出有效数据
        byte[] validData = Arrays.copyOfRange(bytes, 1, bytes.length - 1);
        // 将有效数据转换成实际重量值
        int weight = Integer.parseInt(new String(validData)) / 10;
        CommonWeighbridgeHandler.cacheWeight(weight);
    }

    @Override
    public void handlerAction(String actionCommand) {
        actionCommand = actionCommand.toUpperCase();
        if (StrUtil.equals(actionCommand, WeighbridgeCommonAction.START_READ)) {
            //开始读取地磅数据
            CommonWeighbridgeHandler.startToRead();
        }
    }
}
